//  Created by hao.huang on 2021/11/23 .
//  describe:
import 'package:flutter/material.dart';
import 'package:hhlifecycle_flutter/page_stack.dart';
import 'package:visibility_detector/visibility_detector.dart';
import 'life_state_interface.dart';

abstract class LifeCycleBaseState<W extends StatefulWidget> extends State<W> with LifeStateInterface,WidgetsBindingObserver{
  String tag="";
  bool isDispose=false;///是否销毁
  bool isPause=false;///是否后台运行
  bool isResume=false;///是否可见
  num uid=DateTime.now().microsecondsSinceEpoch;///记录当前页面
  @override
  void initState() {
    super.initState();
    isDispose=false;
    isPause=false;
    isResume=false;
    tag=W.toString();
    onStart();
    WidgetsBinding? widgetsBinding = WidgetsBinding.instance;
    widgetsBinding?.addObserver(this);
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return VisibilityDetector(
      key: Key("${W.toString()}$uid"),
      onVisibilityChanged: (visibilityInfo) {
        var visiblePercentage = visibilityInfo.visibleFraction * 100;
        if(visiblePercentage>99){
          if(!isDispose&&!isResume) {
            onResume();
          }
        }else if(visiblePercentage==0){
          if(!isDispose&&!isPause) {
            onPause();
          }
        }
      },
      child: onBuild(context),
    );
  }


  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    switch(state){
      case AppLifecycleState.resumed:
        if(!isDispose&&!isResume&&PageStack.isCurrentPage(tag,uid)) {
          // debugPrint("=====$tag-$uid:页面resumed========");
          onResume();
        }
        break;
      case AppLifecycleState.paused:
        if(!isDispose&&!isPause) {
          // debugPrint("======$tag-$uid:页面paused========");
          onPause();
        }
        break;
    }
  }

  @override
  void dispose() {
    super.dispose();
    WidgetsBinding.instance?.removeObserver(this);
    isDispose=true;
    onDestory();
  }

  @override
  void onStart() {
    // 页面刚进入
    PageStack.addPage(tag,uid);
    // debugPrint("=======$tag-$uid:onStart()========");
  }

  @override
  void onPause() {
    // 页面停留在后台
    isPause=true;
    isResume=false;
    // debugPrint("=======$tag-$uid:onPause()========");
  }

  @override
  void onResume() {
    // 页面重新可见
    isResume=true;
    isPause=false;
    // debugPrint("=======$tag-$uid:onResume()========");
  }


  @override
  void onDestory() {
    // 页面销毁
    PageStack.removePage(tag,uid);
    // debugPrint("=======$tag:onDestory()========");
  }
}